package com.inkasso.factoring.bocom.vipaccount.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.inkasso.factoring.common.entity.BaseEntity;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 会员账户-基础信息
 * </p>
 *
 * @author wangyafan
 * @since 2024-07-29
 */
@TableName("bocom_vip_account_basic_info")
public class BocomVipAccountBasicInfo extends BaseEntity {

    /**
     * 交通银行客户信息表id
     */
    @NotNull(message = "交通银行客户信息表id不能为空！")
    @TableField("bocom_basic_info_id")
    private Long bocomBasicInfoId;

    /**
     * 账号
     */
    @TableField("account_number")
    private String accountNumber;

    /**
     * 账户名称
     */
    @TableField("account_name")
    private String accountName;

    /**
     * 账户别名
     */
    @TableField("account_alias")
    private String accountAlias;

    /**
     * 账户状态 1-生效 3-失效
     */
    @TableField("account_status")
    private Integer accountStatus;

    /**
     * 授权状态 0-待用户授权 1-银行审核中 2-授权失败（用户拒绝）3-授权失败（银行拒绝）4-授权成功
     */
    @TableField("authorization_status")
    private Integer authorizationStatus;

    /**
     * 渠道
     */
    @TableField("channel")
    private String channel;

    /**
     * 来源
     */
    @TableField("source")
    private String source;

    /**
     * 账户余额
     */
    @TableField("account_balance")
    private BigDecimal accountBalance;

    /**
     * 可用余额
     */
    @TableField("available_balance")
    private BigDecimal availableBalance;

    /**
     * 冻结金额
     */
    @TableField("freeze_amount")
    private BigDecimal freezeAmount;

    /**
     * 开通时间
     */
    @TableField("open_time")
    private Date openTime;

    /**
     * 失效日期
     */
    @TableField("expiring_date")
    private Date expiringDate;

    /**
     * 币种
     */
    @TableField("currency")
    private String currency;


    /**
     * 授权二维码（base64）
     */
    @TableField("qr_code")
    private String qrCode;

    /**
     * 授权二维码生效时间（base64）
     */
    @TableField("qr_code_effect_time")
    private Date qrCodeEffectTime;

    /**
     * 新增会员登记簿的平台流水号
     */
    @TableField("plfm_ser_no")
    private String plfmSerNo;

    /**
     * 登记簿状态 1--可收可付2-销户
     */
    @TableField("chdbok_sts")
    private String chdbokSts;

    /**
     * 付款方账户信息
     */
    @TableField(exist = false)
    private List<BocomVipSilentAccountInfo> silentAccountInfoList = new ArrayList<>();

    /**
     * 删除的静默户的id
     */
    @TableField(exist = false)
    private List<Long> deleteSilentIds;

    /**
     * 绑定的实体账号
     */
    @TableField(exist = false)
    private String bindAcctBankNo;

    /**
     * 付款方账户数量
     */
    @TableField(exist = false)
    private Integer silentAccountNum;

    /**
     * 是否需要重新创建会员登记簿 1 是 0 否
     */
    @TableField(exist = false)
    private Integer isRebuild;

    /**
     * 账户额度限制/其它状态
     */
    @TableField(exist = false)
    private  BocomVipLimitInfo bocomVipLimitInfo;

    /**
     * 客户名称
     */
    @TableField(exist = false)
    private String customerName;

    /**
     * 客户号
     */
    @TableField(exist = false)
    private String customerNumber;

    /**
     * 0企业 1个体工商户 2个人
     */
    @TableField(exist = false)
    private Integer customerType;

    /**
     * 客户来源
     */
    @TableField(exist = false)
    private String customerSource;

    /**
     * 绑定实体卡信息
     */
    @TableField(exist = false)
    private List<BocomVipBindAccountInfo> bindAccountInfoList = new ArrayList<>();

    @TableField(exist = false)
    private String customerStatus;

    /**
     * 认证人
     */
    @TableField(exist = false)
    private String authenticator;

    public List<BocomVipSilentAccountInfo> getSilentAccountInfoList() {
        return silentAccountInfoList;
    }

    public void setSilentAccountInfoList(List<BocomVipSilentAccountInfo> silentAccountInfoList) {
        this.silentAccountInfoList = silentAccountInfoList;
    }

    public String getAccountNumber() {
        return accountNumber;
    }

    public void setAccountNumber(String accountNumber) {
        this.accountNumber = accountNumber;
    }

    public String getAccountName() {
        return accountName;
    }

    public void setAccountName(String accountName) {
        this.accountName = accountName;
    }

    public String getAccountAlias() {
        return accountAlias;
    }

    public void setAccountAlias(String accountAlias) {
        this.accountAlias = accountAlias;
    }

    public Integer getAccountStatus() {
        return accountStatus;
    }

    public void setAccountStatus(Integer accountStatus) {
        this.accountStatus = accountStatus;
    }

    public Integer getAuthorizationStatus() {
        return authorizationStatus;
    }

    public void setAuthorizationStatus(Integer authorizationStatus) {
        this.authorizationStatus = authorizationStatus;
    }

    public String getChannel() {
        return channel;
    }

    public void setChannel(String channel) {
        this.channel = channel;
    }

    public String getSource() {
        return source;
    }

    public void setSource(String source) {
        this.source = source;
    }

    public BigDecimal getAccountBalance() {
        return accountBalance;
    }

    public void setAccountBalance(BigDecimal accountBalance) {
        this.accountBalance = accountBalance;
    }

    public BigDecimal getAvailableBalance() {
        return availableBalance;
    }

    public void setAvailableBalance(BigDecimal availableBalance) {
        this.availableBalance = availableBalance;
    }

    public BigDecimal getFreezeAmount() {
        return freezeAmount;
    }

    public void setFreezeAmount(BigDecimal freezeAmount) {
        this.freezeAmount = freezeAmount;
    }

    public Date getOpenTime() {
        return openTime;
    }

    public void setOpenTime(Date openTime) {
        this.openTime = openTime;
    }

    public Date getExpiringDate() {
        return expiringDate;
    }

    public void setExpiringDate(Date expiringDate) {
        this.expiringDate = expiringDate;
    }

    public String getCurrency() {
        return currency;
    }

    public void setCurrency(String currency) {
        this.currency = currency;
    }

    public Long getBocomBasicInfoId() {
        return bocomBasicInfoId;
    }

    public void setBocomBasicInfoId(Long bocomBasicInfoId) {
        this.bocomBasicInfoId = bocomBasicInfoId;
    }

    public Integer getSilentAccountNum() {
        return silentAccountNum;
    }

    public void setSilentAccountNum(Integer silentAccountNum) {
        this.silentAccountNum = silentAccountNum;
    }

    public String getBindAcctBankNo() {
        return bindAcctBankNo;
    }

    public void setBindAcctBankNo(String bindAcctBankNo) {
        this.bindAcctBankNo = bindAcctBankNo;
    }

    public String getQrCode() {
        return qrCode;
    }

    public void setQrCode(String qrCode) {
        this.qrCode = qrCode;
    }

    public Date getQrCodeEffectTime() {
        return qrCodeEffectTime;
    }

    public void setQrCodeEffectTime(Date qrCodeEffectTime) {
        this.qrCodeEffectTime = qrCodeEffectTime;
    }

    public BocomVipLimitInfo getBocomVipLimitInfo() {
        return bocomVipLimitInfo;
    }

    public void setBocomVipLimitInfo(BocomVipLimitInfo bocomVipLimitInfo) {
        this.bocomVipLimitInfo = bocomVipLimitInfo;
    }

    public Integer getIsRebuild() {
        return isRebuild;
    }

    public void setIsRebuild(Integer isRebuild) {
        this.isRebuild = isRebuild;
    }

    public String getCustomerName() {
        return customerName;
    }

    public void setCustomerName(String customerName) {
        this.customerName = customerName;
    }

    public String getCustomerNumber() {
        return customerNumber;
    }

    public void setCustomerNumber(String customerNumber) {
        this.customerNumber = customerNumber;
    }

    public Integer getCustomerType() {
        return customerType;
    }

    public void setCustomerType(Integer customerType) {
        this.customerType = customerType;
    }

    public String getCustomerSource() {
        return customerSource;
    }

    public void setCustomerSource(String customerSource) {
        this.customerSource = customerSource;
    }

    public List<BocomVipBindAccountInfo> getBindAccountInfoList() {
        return bindAccountInfoList;
    }

    public void setBindAccountInfoList(List<BocomVipBindAccountInfo> bindAccountInfoList) {
        this.bindAccountInfoList = bindAccountInfoList;
    }

    public List<Long> getDeleteSilentIds() {
        return deleteSilentIds;
    }

    public void setDeleteSilentIds(List<Long> deleteSilentIds) {
        this.deleteSilentIds = deleteSilentIds;
    }

    public String getCustomerStatus() {
        return customerStatus;
    }

    public void setCustomerStatus(String customerStatus) {
        this.customerStatus = customerStatus;
    }

    public String getPlfmSerNo() {
        return plfmSerNo;
    }

    public void setPlfmSerNo(String plfmSerNo) {
        this.plfmSerNo = plfmSerNo;
    }

    public String getChdbokSts() {
        return chdbokSts;
    }

    public void setChdbokSts(String chdbokSts) {
        this.chdbokSts = chdbokSts;
    }

    public String getAuthenticator() {
        return authenticator;
    }

    public void setAuthenticator(String authenticator) {
        this.authenticator = authenticator;
    }

    @Override
    public String toString() {
        return "BocomVipAccountBasicInfo{" +
                "bocomBasicInfoId=" + bocomBasicInfoId +
                ", accountNumber=" + accountNumber +
                ", accountName=" + accountName +
                ", accountAlias=" + accountAlias +
                ", accountStatus=" + accountStatus +
                ", authorizationStatus=" + authorizationStatus +
                ", channel=" + channel +
                ", source=" + source +
                ", accountBalance=" + accountBalance +
                ", availableBalance=" + availableBalance +
                ", freezeAmount=" + freezeAmount +
                ", openTime=" + openTime +
                ", expiringDate=" + expiringDate +
                ", currency=" + currency +
                "}";
    }
}
