package com.inkasso.factoring.bocom.vipaccount.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.inkasso.adapter.exception.BusinessException;
import com.inkasso.adapter.util.CodeUtil;
import com.inkasso.adapter.web.LoginUserHolder;
import com.inkasso.factoring.bocom.register.entity.BocomBasicInfo;
import com.inkasso.factoring.bocom.register.entity.BocomLegalPersonInfo;
import com.inkasso.factoring.bocom.register.entity.BocomShareholderActualController;
import com.inkasso.factoring.bocom.register.service.IBocomBasicInfoService;
import com.inkasso.factoring.bocom.register.service.IBocomLegalPersonInfoService;
import com.inkasso.factoring.bocom.register.service.IBocomShareholderActualControllerService;
import com.inkasso.factoring.bocom.vipaccount.dao.BocomVipAccountBasicInfoMapper;
import com.inkasso.factoring.bocom.vipaccount.entity.BocomVipAccountBasicInfo;
import com.inkasso.factoring.bocom.vipaccount.entity.BocomVipBindAccountInfo;
import com.inkasso.factoring.bocom.vipaccount.entity.BocomVipLimitInfo;
import com.inkasso.factoring.bocom.vipaccount.entity.BocomVipSilentAccountInfo;
import com.inkasso.factoring.bocom.vipaccount.entity.vo.BocomVipAccountListVo;
import com.inkasso.factoring.bocom.vipaccount.service.IBocomVipAccountBasicInfoService;
import com.inkasso.factoring.bocom.vipaccount.service.IBocomVipBindAccoutInfoService;
import com.inkasso.factoring.bocom.vipaccount.service.IBocomVipLimitInfoService;
import com.inkasso.factoring.bocom.vipaccount.service.IBocomVipSilentAccoutInfoService;
import com.inkasso.factoring.common.entity.ResponseJson;
import com.inkasso.factoring.common.service.SerialNumberGenerator;
import com.inkasso.factoring.common.util.DateUtil;
import com.inkasso.factoring.dict.entity.Dictionary;
import com.inkasso.factoring.dict.service.DictionaryService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 会员账户-基础信息 服务实现类
 * </p>
 *
 * @author wangyafan
 * @since 2024-08-06
 */
@Service
public class BocomVipAccountBasicInfoServiceImpl extends ServiceImpl<BocomVipAccountBasicInfoMapper, BocomVipAccountBasicInfo> implements IBocomVipAccountBasicInfoService {

    @Autowired
    private IBocomBasicInfoService bocomBasicInfoService;
    @Autowired
    private IBocomVipSilentAccoutInfoService bocomVipSilentAccoutInfoService;
    @Autowired
    private IBocomVipBindAccoutInfoService bocomVipBindAccountInfoService;
    @Autowired
    private IBocomVipLimitInfoService bocomVipLimitInfoService;
    @Autowired
    private RestTemplate restTemplate;
    @Value("${trade.bocomVipUpdateUrl}")
    private String bocomVipUpdateUrl;
    @Autowired
    private DictionaryService dictionaryService;
    @Value("${trade.bocomVipRefreshUrl}")
    private String bocomVipRefreshUrl;
    @Value("${trade.bocomVipResetQrCodeUrl}")
    private String bocomVipResetQrCodeUrl;
    @Value("${trade.bocomVipResetActvtAmtUrl}")
    private String bocomVipResetActvtAmtUrl;
    @Value("${trade.bocomVipBindDoUpdateUrl}")
    private String bocomVipBindDoUpdateUrl;
    @Autowired
    private SerialNumberGenerator serialNumberGenerator;
    @Autowired
    private IBocomLegalPersonInfoService bocomLegalPersonInfoService;
    @Autowired
    private IBocomShareholderActualControllerService bocomShareholderActualControllerService;

    @Override
    public List<BocomVipAccountListVo> findList() {

        QueryWrapper<BocomBasicInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_create", LoginUserHolder.get().getUserName());
        queryWrapper.orderByDesc("gmt_modified");
        List<BocomBasicInfo> bocomBasicInfos = bocomBasicInfoService.list(queryWrapper);
        List<BocomVipAccountListVo> voList = new ArrayList<>();

        //按照账户信息分组
        if (CollectionUtils.isNotEmpty(bocomBasicInfos)) {

            bocomBasicInfos.forEach(bocomBasicInfo -> {
                BocomVipAccountListVo vo = new BocomVipAccountListVo();
                vo.setBocomBasicInfoId(bocomBasicInfo.getId());
                vo.setCustomerNumber(bocomBasicInfo.getCustomerNumber());
                vo.setCustomerName(bocomBasicInfo.getCustomerName());
                Dictionary customerStatusDic = dictionaryService.getByCode(bocomBasicInfo.getCustomerStatus());
                if (customerStatusDic != null) {
                    vo.setCustomerStatus(customerStatusDic.getName());
                }

                if (bocomBasicInfo.getCustomerType() != null) {
                    if (bocomBasicInfo.getCustomerType() == 0) {
                        BocomLegalPersonInfo legalPersonInfo = bocomLegalPersonInfoService.lambdaQuery()
                                .eq(BocomLegalPersonInfo::getBocomBasicInfoId, bocomBasicInfo.getId())
                                .one();
                        if (legalPersonInfo != null) {
                            vo.setAuthenticator(legalPersonInfo.getName());
                        }
                    }
                    if (bocomBasicInfo.getCustomerType() == 1) {
                        List<BocomShareholderActualController> shareholderActualControllerList = bocomShareholderActualControllerService.lambdaQuery()
                                .eq(BocomShareholderActualController::getBocomBasicInfoId, bocomBasicInfo.getId())
                                .orderByDesc(BocomShareholderActualController::getGmtCreate)
                                .list();
                        if (CollectionUtil.isNotEmpty(shareholderActualControllerList)) {
                            BocomShareholderActualController bocomShareholderActualController = shareholderActualControllerList.get(0);
                            if (bocomShareholderActualController != null) {
                                vo.setAuthenticator(bocomShareholderActualController.getName());
                            }
                        }
                    }
                    if (bocomBasicInfo.getCustomerType() == 2) {
                        vo.setAuthenticator(bocomBasicInfo.getCustomerName());
                    }
                }

                List<BocomVipAccountBasicInfo> vipList = lambdaQuery()
                        .eq(BocomVipAccountBasicInfo::getBocomBasicInfoId, bocomBasicInfo.getId())
                        .list();
                if (CollectionUtils.isNotEmpty(vipList)) {
                    vipList.forEach(vipAccountBasicInfo -> {
                        //标识是否需要重新创建会员登记簿
                        Date afterFiveDaysDate = DateUtil.addDays(vipAccountBasicInfo.getGmtModified(), 5);
                        if (new Date().before(afterFiveDaysDate)) {
                            vipAccountBasicInfo.setIsRebuild(0);
                        } else {
                            vipAccountBasicInfo.setIsRebuild(1);
                        }
                        //客户状态
                        if (customerStatusDic != null) {
                            vo.setCustomerStatus(customerStatusDic.getName());
                        }
                        //静默户数量
                        Integer count = Math.toIntExact(bocomVipSilentAccoutInfoService.lambdaQuery()
                                .eq(BocomVipSilentAccountInfo::getBocomVipAccountBasicInfoId, vipAccountBasicInfo.getId())
                                .count());
                        vipAccountBasicInfo.setSilentAccountNum(count);

                        //绑定银行卡
                        List<BocomVipBindAccountInfo> bindAccountInfos = bocomVipBindAccountInfoService.lambdaQuery()
                                .eq(BocomVipBindAccountInfo::getBocomVipAccountBasicInfoId, vipAccountBasicInfo.getId())
                                .orderByDesc(BocomVipBindAccountInfo::getGmtCreate)
                                .list();
                        if (CollectionUtil.isNotEmpty(bindAccountInfos)) {
                            vipAccountBasicInfo.setBindAccountInfoList(bindAccountInfos);
                        }
                    });
                    vo.setVipAccountBasicInfoList(vipList);
                }
                voList.add(vo);
            });
        }
        return voList;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BocomVipAccountBasicInfo doAdd(Long bocomBasicInfoId) {

        if (bocomBasicInfoId == null) {
            throw new BusinessException(CodeUtil.FAIL, "开户id不能为空！");
        }
        BocomBasicInfo basicInfo = bocomBasicInfoService.getById(bocomBasicInfoId);
        if (basicInfo == null) {
            throw new BusinessException(CodeUtil.FAIL, "未查询到客户信息！");
        }
        BocomVipAccountBasicInfo vipAccountBasicInfo = new BocomVipAccountBasicInfo();
        vipAccountBasicInfo.setBocomBasicInfoId(basicInfo.getId());
        vipAccountBasicInfo.setAccountName(basicInfo.getCustomerName());
        vipAccountBasicInfo.setAccountAlias(basicInfo.getCustomerName());
        vipAccountBasicInfo.setSource("贸理通");
        vipAccountBasicInfo.setCurrency("CNY");
        ResponseJson responseJson;
        try {
            responseJson = restTemplate.postForObject(bocomVipUpdateUrl, vipAccountBasicInfo, ResponseJson.class);
            if (!responseJson.getCode().equals(200)) {
                throw new BusinessException(CodeUtil.FAIL, responseJson.getMsg().toString());
            }

        } catch (Exception e) {
            throw new BusinessException(CodeUtil.FAIL, e.getMessage());
        }
        BocomVipAccountBasicInfo newBasicInfo = JSONObject.parseObject(JSONObject.toJSONString(responseJson.getData()), BocomVipAccountBasicInfo.class);
        if (newBasicInfo.getId() == null) {
            if (newBasicInfo.getAccountStatus() != null && newBasicInfo.getAccountStatus() == 1) {
                newBasicInfo.setOpenTime(new Date());
            }
            save(newBasicInfo);
            if (newBasicInfo.getBocomVipLimitInfo() != null) {
                BocomVipLimitInfo bocomVipLimitInfo = newBasicInfo.getBocomVipLimitInfo();
                bocomVipLimitInfo.setBocomVipAccountBasicInfoId(newBasicInfo.getId());
                bocomVipLimitInfoService.save(bocomVipLimitInfo);
            }
        }
        if (basicInfo.getCustomerType() != null) {
            if (0 == newBasicInfo.getCustomerType()) {
                BocomLegalPersonInfo legalPersonInfo = bocomLegalPersonInfoService.lambdaQuery()
                        .eq(BocomLegalPersonInfo::getBocomBasicInfoId, basicInfo.getId())
                        .one();
                if (legalPersonInfo != null) {
                    newBasicInfo.setAuthenticator(legalPersonInfo.getName());
                }
            }
            if (1 == newBasicInfo.getCustomerType()) {
                List<BocomShareholderActualController> shareholderActualControllerList = bocomShareholderActualControllerService.lambdaQuery()
                        .eq(BocomShareholderActualController::getBocomBasicInfoId, basicInfo.getId())
                        .orderByDesc(BocomShareholderActualController::getGmtCreate)
                        .list();
                if (cn.hutool.core.collection.CollectionUtil.isNotEmpty(shareholderActualControllerList)) {
                    BocomShareholderActualController bocomShareholderActualController = shareholderActualControllerList.get(0);
                    if (bocomShareholderActualController != null) {
                        newBasicInfo.setAuthenticator(bocomShareholderActualController.getName());
                    }
                }
            }
            if (2 == newBasicInfo.getCustomerType()) {
                newBasicInfo.setAuthenticator(basicInfo.getCustomerName());
            }
        }

        return newBasicInfo;
    }

    @Override
    public BocomVipAccountBasicInfo getDetail(Long id) {
        BocomVipAccountBasicInfo basicInfo = getById(id);
        if (basicInfo != null) {
            BocomBasicInfo bocomBasicInfo = bocomBasicInfoService.getDetail(basicInfo.getBocomBasicInfoId());
            //客户信息
            basicInfo.setCustomerName(bocomBasicInfo.getCustomerName());
            basicInfo.setCustomerName(bocomBasicInfo.getCustomerName());
            basicInfo.setCustomerNumber(bocomBasicInfo.getCustomerNumber());
            basicInfo.setCustomerSource(bocomBasicInfo.getCustomerSource());
            basicInfo.setCustomerType(bocomBasicInfo.getCustomerType());
            List<BocomVipSilentAccountInfo> silentAccountInfos = bocomVipSilentAccoutInfoService.lambdaQuery()
                    .eq(BocomVipSilentAccountInfo::getBocomVipAccountBasicInfoId, id)
                    .list();
            if (CollectionUtil.isNotEmpty(silentAccountInfos)) {
                basicInfo.setSilentAccountInfoList(silentAccountInfos);
            }

            //绑定银行卡信息
            List<BocomVipBindAccountInfo> bindAccountInfoList = bocomVipBindAccountInfoService.lambdaQuery()
                    .eq(BocomVipBindAccountInfo::getBocomVipAccountBasicInfoId, id)
                    .list();
            if (CollectionUtil.isNotEmpty(bindAccountInfoList)) {
                basicInfo.setBindAccountInfoList(bindAccountInfoList);
            }

            //账户额度限制/其它状态
            BocomVipLimitInfo vipLimitInfo = bocomVipLimitInfoService.lambdaQuery().eq(BocomVipLimitInfo::getBocomVipAccountBasicInfoId, id).one();
            basicInfo.setBocomVipLimitInfo(vipLimitInfo);
        }
        return basicInfo;
    }

    @Override
    public void doUpdate(BocomVipAccountBasicInfo basicInfo) {
        if (basicInfo.getId() == null) {
            throw new BusinessException(CodeUtil.FAIL, "会员登记簿id不能为空！");
        }
        BocomVipAccountBasicInfo oldBasicInfo = getById(basicInfo.getId());
        basicInfo.setAccountNumber(oldBasicInfo.getAccountNumber());
        basicInfo.setBocomBasicInfoId(oldBasicInfo.getBocomBasicInfoId());
        ResponseJson responseJson = null;
        if (CollectionUtil.isNotEmpty(basicInfo.getSilentAccountInfoList())) {
            List<BocomVipSilentAccountInfo> newSilentAccountInfoList = basicInfo.getSilentAccountInfoList();
            for (BocomVipSilentAccountInfo newSilentAccountInfo : newSilentAccountInfoList) {

                ArrayList<BocomVipSilentAccountInfo> silentAccountInfos = new ArrayList<>();
                silentAccountInfos.add(newSilentAccountInfo);
                oldBasicInfo.setSilentAccountInfoList(silentAccountInfos);
                try {
                    responseJson = restTemplate.postForObject(bocomVipUpdateUrl, basicInfo, ResponseJson.class);
                    if (!responseJson.getCode().equals(200)) {
                        throw new BusinessException(CodeUtil.FAIL, responseJson.getMsg().toString());
                    }

                } catch (Exception e) {
                    throw new BusinessException(CodeUtil.FAIL, e.getMessage());
                }
                //静默户入库
                newSilentAccountInfo.setBocomVipAccountBasicInfoId(oldBasicInfo.getId());
                bocomVipSilentAccoutInfoService.save(newSilentAccountInfo);
            }

            if (CollectionUtil.isNotEmpty(basicInfo.getDeleteSilentIds())) {
                //删除会员登记簿
                for (Long deleteSilentId : basicInfo.getDeleteSilentIds()) {
                    List<Long> ids = new ArrayList<>();
                    ids.add(deleteSilentId);
                    oldBasicInfo.setDeleteSilentIds(ids);
                    try {
                        responseJson = restTemplate.postForObject(bocomVipUpdateUrl, basicInfo, ResponseJson.class);
                        if (!responseJson.getCode().equals(200)) {
                            throw new BusinessException(CodeUtil.FAIL, responseJson.getMsg().toString());
                        }

                    } catch (Exception e) {
                        throw new BusinessException(CodeUtil.FAIL, e.getMessage());
                    }
                    bocomVipSilentAccoutInfoService.removeById(deleteSilentId);
                }

            }

            if (responseJson != null){
                BocomVipAccountBasicInfo newBasicInfo = JSONObject.parseObject(JSONObject.toJSONString(responseJson.getData()), BocomVipAccountBasicInfo.class);

                updateById(newBasicInfo);
                if (newBasicInfo.getBocomVipLimitInfo() != null) {
                    BocomVipLimitInfo bocomVipLimitInfo = newBasicInfo.getBocomVipLimitInfo();
                    BocomVipLimitInfo oldLimitInfo = bocomVipLimitInfoService.lambdaQuery()
                            .eq(BocomVipLimitInfo::getBocomVipAccountBasicInfoId, newBasicInfo.getId())
                            .one();
                    bocomVipLimitInfo.setBocomVipAccountBasicInfoId(newBasicInfo.getId());
                    bocomVipLimitInfo.setId(oldLimitInfo.getId());
                    bocomVipLimitInfoService.updateById(bocomVipLimitInfo);

                }
            }

        }
    }

    @Override
    public void refreshBocomVipBasic(Long id) {

        if (id == null) {
            throw new BusinessException(CodeUtil.FAIL, "id不能为空！");
        }
        try {
            ResponseJson responseJson = restTemplate.getForObject(bocomVipRefreshUrl + "?id=" + id, ResponseJson.class);
            if (!responseJson.getCode().equals(200)) {
                throw new BusinessException(CodeUtil.FAIL, responseJson.getMsg().toString());
            }
        } catch (Exception e) {
            throw new BusinessException(CodeUtil.FAIL, "刷新登记簿信息失败！" + e.getMessage());
        }


    }

    @Override
    public void resetUrl(Long id) {
        BocomVipAccountBasicInfo basicInfo = getById(id);
        if (basicInfo != null) {
            ResponseJson responseJson;
            try {
                responseJson = restTemplate.postForObject(bocomVipResetQrCodeUrl, basicInfo, ResponseJson.class);
                if (!responseJson.getCode().equals(200)) {
                    throw new BusinessException(CodeUtil.FAIL, responseJson.getMsg().toString());
                }
            } catch (Exception e) {
                throw new BusinessException(CodeUtil.FAIL, e.getMessage());
            }
            BocomVipAccountBasicInfo vipAccountBasicInfo = JSONObject.parseObject(JSONObject.toJSONString(responseJson.getData()), BocomVipAccountBasicInfo.class);
            lambdaUpdate().eq(BocomVipAccountBasicInfo::getId, vipAccountBasicInfo.getId())
                    .set(BocomVipAccountBasicInfo::getQrCode, vipAccountBasicInfo.getQrCode())
                    .set(BocomVipAccountBasicInfo::getQrCodeEffectTime, vipAccountBasicInfo.getQrCodeEffectTime())
                    .update();

        }
    }

    @Override
    public void updateBindAccount(BocomVipBindAccountInfo bindAccountInfo) {
        if (bindAccountInfo != null) {
            if (bindAccountInfo.getId() == null) {
                bindAccountInfo.setOprTp("0");
            } else {
                bindAccountInfo.setOprTp("1");
            }
            ResponseJson responseJson;
            try {
                responseJson = restTemplate.postForObject(bocomVipBindDoUpdateUrl, bindAccountInfo, ResponseJson.class);
                if (!responseJson.getCode().equals(200)) {
                    throw new BusinessException(CodeUtil.FAIL, responseJson.getMsg().toString());
                }
            } catch (Exception e) {
                throw new BusinessException(CodeUtil.FAIL, e.getMessage());
            }
            BocomVipBindAccountInfo newBindAccountInfo = JSONObject.parseObject(JSONObject.toJSONString(responseJson.getData()), BocomVipBindAccountInfo.class);
            if (newBindAccountInfo.getId() == null) {
                //如果返回的实体卡状态是  入金激活待确认
                if (newBindAccountInfo.getBindSts() != null && newBindAccountInfo.getBindSts() == 2) {
                    bindAccountInfo.setMoneyConfirmStartTime(new Date());
                }
                bocomVipBindAccountInfoService.save(newBindAccountInfo);
            } else {
                bocomVipBindAccountInfoService.removeById(newBindAccountInfo);
            }

        }
    }

    @Override
    public void resetAmount(Long id) {

        BocomVipBindAccountInfo bindAccountInfo = bocomVipBindAccountInfoService.getById(id);
        if (bindAccountInfo != null) {
            bindAccountInfo.setOprTp("2");
            ResponseJson responseJson;
            try {
                responseJson = restTemplate.postForObject(bocomVipResetActvtAmtUrl, bindAccountInfo, ResponseJson.class);
                if (!responseJson.getCode().equals(200)) {
                    throw new BusinessException(CodeUtil.FAIL, responseJson.getMsg().toString());
                }

            } catch (Exception e) {
                throw new BusinessException(CodeUtil.FAIL, e.getMessage());
            }

            BocomVipBindAccountInfo newBindAccount = JSONObject.parseObject(JSONObject.toJSONString(responseJson.getData()), BocomVipBindAccountInfo.class);
            if (newBindAccount != null) {
                bocomVipBindAccountInfoService.lambdaUpdate()
                        .eq(BocomVipBindAccountInfo::getId, newBindAccount.getId())
                        .set(BocomVipBindAccountInfo::getActvtAmt, newBindAccount.getActvtAmt())
                        .set(BocomVipBindAccountInfo::getMoneyConfirmStartTime, newBindAccount.getMoneyConfirmStartTime())
                        .update();
            }
        }

    }


}
