package com.inkasso.factoring.bocom.vipaccount.web;

import com.inkasso.factoring.bocom.vipaccount.entity.BocomVipAccountBasicInfo;
import com.inkasso.factoring.bocom.vipaccount.entity.BocomVipBindAccountInfo;
import com.inkasso.factoring.bocom.vipaccount.entity.vo.BocomVipAccountListVo;
import com.inkasso.factoring.bocom.vipaccount.service.IBocomVipAccountBasicInfoService;
import com.inkasso.factoring.common.entity.ResponseJson;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description TODO
 * @Author Yafan Wang
 * @Date 2024/8/9 9:10
 */
@RestController
public class BocomVipAccountController {

    @Autowired
    private IBocomVipAccountBasicInfoService bocomVipAccountBasicInfoService;


    /**
     * 新增会员登记簿
     *
     * @param bocomBasicInfoId
     * @return
     */
    @GetMapping("bocomVipAccount/doAdd")
    public ResponseJson doAdd(@RequestParam Long bocomBasicInfoId) {
        return new ResponseJson(bocomVipAccountBasicInfoService.doAdd(bocomBasicInfoId));
    }


    /**
     * 列表
     *
     * @return
     */
    @GetMapping("bocomVipAccount/list")
    public ResponseJson<List<BocomVipAccountListVo>> list() {
        return new ResponseJson(bocomVipAccountBasicInfoService.findList());
    }

    /**
     * 修改会员登记簿（新增/删除静默户）
     *
     * @param basicInfo
     * @return
     */
    @PostMapping("bocomVipAccount/doUpdate")
    public ResponseJson doUpdate(@RequestBody BocomVipAccountBasicInfo basicInfo) {
        bocomVipAccountBasicInfoService.doUpdate(basicInfo);
        return new ResponseJson("操作成功！");
    }

    /**
     * 获取详情
     *
     * @return
     */
    @GetMapping("bocomVipAccount/getDetail")
    public ResponseJson<BocomVipAccountBasicInfo> getDetail(@RequestParam Long id) {
        return new ResponseJson(bocomVipAccountBasicInfoService.getDetail(id));
    }

    /**
     * 更新会员登记簿信息
     *
     * @return
     */
    @GetMapping("bocomVipAccount/refreshBocomVipBasic")
    public ResponseJson refreshBocomVipBasic(@RequestParam Long id) {
        bocomVipAccountBasicInfoService.refreshBocomVipBasic(id);
        return new ResponseJson("操作成功！");
    }

    /**
     *重置授权二维码
     *
     * @param id
     * @return
     */
    @GetMapping("bocomVipAccount/resetUrl")
    public ResponseJson resetUrl(@RequestParam Long id) {
        bocomVipAccountBasicInfoService.resetUrl(id);
        return new ResponseJson("操作成功！");
    }


    /**
     * 修改绑定实体卡信息
     *
     * @param bindAccountInfo
     * @return
     */
    @PostMapping("bocomVipAccount/updateBindAccount")
    public ResponseJson updateBindAccount(@RequestBody BocomVipBindAccountInfo bindAccountInfo) {
        bocomVipAccountBasicInfoService.updateBindAccount(bindAccountInfo);
        return new ResponseJson("操作成功！");
    }


    /**
     * 重设入金激活金额
     *
     * @param id
     * @return
     */
    @GetMapping("bocomVipAccount/resetActvtAmt")
    public ResponseJson resetActvtAmt(@RequestParam Long id) {
        bocomVipAccountBasicInfoService.resetAmount(id);
        return new ResponseJson("操作成功！");
    }


}
